<?php
/*
Plugin Name: Custom Media Display
Description: Adds a custom media display under the Media Library with sorting and deletion options.
Version: 1.15
Author: EasyToolsWP.com
*/

function cmd_get_common_words($text1, $text2) {
    $stop_words = array('the', 'and', 'or', 'of', 'a', 'an', 'in', 'on', 'for', 'to', 'with', 'at', 'by', 'og', 'på', 'nu', 'til', 'fra', 'en', 'to', 'eller', 'copy');
    $sanitize_text = function($text) use ($stop_words) {
        $text = strtolower($text);
        $text = preg_replace('/[^\w\s]/', '', $text);
        $words = array_diff(explode(' ', $text), $stop_words);
        return array_unique($words);
    };

    $words1 = $sanitize_text($text1);
    $words2 = $sanitize_text($text2);

    $common_words = array_intersect($words1, $words2);

    return implode(', ', $common_words);
}

add_action('admin_menu', 'cmd_add_custom_media_page');
function cmd_add_custom_media_page() {
    add_submenu_page(
        'upload.php', // Skift til 'upload.php' for Mediebiblioteket
        'Custom Media Display',
        'Custom Media Display',
        'manage_options',
        'custom-media-display',
        'cmd_render_custom_media_page'
    );
}

function cmd_render_custom_media_page() {
    if ($_SERVER['REQUEST_METHOD'] === 'POST' && isset($_POST['cmd_media_display_settings'])) {
        check_admin_referer('cmd_media_display_action', 'cmd_media_display_nonce');

        update_option('cmd_media_display_settings', $_POST['cmd_media_display_settings']);
        echo '<div class="updated notice is-dismissible"><p>Settings saved.</p></div>';
    }
    ?>
    <div class="wrap">
        <h1>Custom Media Display</h1>
        <form method="post" action="">
            <?php wp_nonce_field('cmd_media_display_action', 'cmd_media_display_nonce'); ?>
            <?php
            $options = get_option('cmd_media_display_settings', array(
                'show_image'    => '',
                'rows_per_page' => 50,
            ));
            $rows_per_page = isset($options['rows_per_page']) ? intval($options['rows_per_page']) : 50;
            $show_image = isset($options['show_image']) ? $options['show_image'] : '';
            ?>
            <h2>Settings</h2>
            <table class="form-table">
                <tr>
                    <th scope="row">Show Image in Rows</th>
                    <td>
                        <input type="checkbox" name="cmd_media_display_settings[show_image]" value="1" <?php checked($show_image, '1'); ?> />
                    </td>
                </tr>
                <tr>
                    <th scope="row">Rows Per Page</th>
                    <td>
                        <select name="cmd_media_display_settings[rows_per_page]">
                            <option value="10" <?php selected($rows_per_page, 10); ?>>10</option>
                            <option value="25" <?php selected($rows_per_page, 25); ?>>25</option>
                            <option value="50" <?php selected($rows_per_page, 50); ?>>50</option>
                            <option value="75" <?php selected($rows_per_page, 75); ?>>75</option>
                            <option value="100" <?php selected($rows_per_page, 100); ?>>100</option>
                            <option value="250" <?php selected($rows_per_page, 250); ?>>250</option>
                            <option value="500" <?php selected($rows_per_page, 500); ?>>500 (!)</option>
                            <option value="1000" <?php selected($rows_per_page, 1000); ?>>1000 (!!)</option>
                        </select>
                    </td>
                </tr>
            </table>
            <?php submit_button('Save Settings'); ?>
        </form>

        <?php cmd_display_custom_media_list(); ?>
    </div>
    <?php
}

function cmd_display_custom_media_list() {
    $options = get_option('cmd_media_display_settings');
    $rows_per_page = isset($options['rows_per_page']) ? intval($options['rows_per_page']) : 50;
    $show_image = isset($options['show_image']) ? $options['show_image'] : '';

    if (isset($_POST['action'])) {
        check_admin_referer('cmd_media_display_action', 'cmd_media_display_nonce');

        if ($_POST['action'] == 'bulk_delete') {
            $ids = isset($_POST['media_ids']) ? $_POST['media_ids'] : array();
            foreach ($ids as $id) {
                wp_delete_attachment($id, true);
            }
            echo '<div class="updated notice is-dismissible"><p>Selected media items deleted.</p></div>';
        } elseif ($_POST['action'] == 'bulk_update_alt_text') {
            $alt_texts = isset($_POST['alt_text']) ? $_POST['alt_text'] : array();
            foreach ($alt_texts as $id => $alt_text) {
                update_post_meta($id, '_wp_attachment_image_alt', sanitize_text_field($alt_text));
            }
            echo '<div class="updated notice is-dismissible"><p>Alt texts updated.</p></div>';
        } elseif ($_POST['action'] == 'bulk_make_filename_alt_text') {
            $ids = isset($_POST['media_ids']) ? $_POST['media_ids'] : array();
            foreach ($ids as $id) {
                $filename = get_post_meta($id, '_wp_attached_file', true);
                $alt_text = pathinfo($filename, PATHINFO_FILENAME);
                $alt_text = str_replace(array('-', '_'), ' ', $alt_text);
                update_post_meta($id, '_wp_attachment_image_alt', sanitize_text_field($alt_text));
            }
            echo '<div class="updated notice is-dismissible"><p>Alt texts updated based on filenames.</p></div>';
        }
    }

    $orderby = isset($_GET['orderby']) ? sanitize_text_field($_GET['orderby']) : 'date';
    $order = isset($_GET['order']) ? sanitize_text_field($_GET['order']) : 'DESC';
    $paged = isset($_GET['paged']) ? intval($_GET['paged']) : 1;
    $filter_type = isset($_GET['filter_type']) ? sanitize_text_field($_GET['filter_type']) : '';
    $search = isset($_GET['s']) ? sanitize_text_field($_GET['s']) : '';
    $date_filter = isset($_GET['date_filter']) ? sanitize_text_field($_GET['date_filter']) : '';

    $date_query = array();
    if ($date_filter == 'last_24_hours') {
        $date_query = array('after' => '1 day ago');
    } elseif ($date_filter == 'last_48_hours') {
        $date_query = array('after' => '2 days ago');
    } elseif ($date_filter == 'this_week') {
        $date_query = array('after' => '1 week ago');
    } elseif ($date_filter == 'this_month') {
        $date_query = array('after' => 'first day of this month');
    } elseif ($date_filter == 'last_month') {
        $date_query = array('after' => 'first day of last month', 'before' => 'first day of this month');
    } elseif ($date_filter) {
        $date_query = array('after' => $date_filter . ' 00:00:00', 'before' => $date_filter . ' 23:59:59');
    }

    $args = array(
        'post_type'      => 'attachment',
        'posts_per_page' => $rows_per_page,
        'post_status'    => 'inherit',
        'orderby'        => $orderby,
        'order'          => $order,
        'paged'          => $paged,
        's'              => $search,
        'date_query'     => $date_query
    );

    if (!empty($filter_type)) {
        $args['post_mime_type'] = $filter_type;
    }

    $query = new WP_Query($args);

    $total_images = $query->found_posts;

    $file_types = array();
    foreach ($query->posts as $post) {
        $mime_type = get_post_mime_type($post->ID);
        if (!in_array($mime_type, $file_types)) {
            $file_types[] = $mime_type;
        }
    }

    $all_file_types = get_posts(array(
        'post_type'      => 'attachment',
        'posts_per_page' => -1,
        'post_status'    => 'inherit',
        'fields'         => 'ids'
    ));
    $file_types = array();
    foreach ($all_file_types as $file_id) {
        $mime_type = get_post_mime_type($file_id);
        if (!in_array($mime_type, $file_types)) {
            $file_types[] = $mime_type;
        }
    }
    ?>
    <div class="wrap">
        <form method="get" action="">
            <input type="hidden" name="page" value="custom-media-display">
            <div class="tablenav top">
                <div class="alignleft actions">
                    <input type="text" name="s" value="<?php echo esc_attr($search); ?>" placeholder="Search">
                    <select name="filter_type">
                        <option value="">Show All</option>
                        <?php
                        $mime_types = array(
                            'application/x-gzip' => 'Archives only',
                            'image/jpeg' => 'JPEG Images',
                            'image/png' => 'PNG Images',
                            // Tilføj flere MIME-typer her
                        );
                        foreach ($mime_types as $mime_type => $label) {
                            echo '<option value="' . esc_attr($mime_type) . '"' . selected($filter_type, $mime_type, false) . '>' . esc_html($label) . '</option>';
                        }
                        ?>
                    </select>
                    <select name="date_filter">
                        <option value="">All Dates</option>
                        <option value="last_24_hours" <?php selected($date_filter, 'last_24_hours'); ?>>Last 24 Hours</option>
                        <option value="last_48_hours" <?php selected($date_filter, 'last_48_hours'); ?>>Last 48 Hours</option>
                        <option value="this_week" <?php selected($date_filter, 'this_week'); ?>>This Week</option>
                        <option value="this_month" <?php selected($date_filter, 'this_month'); ?>>This Month</option>
                        <option value="last_month" <?php selected($date_filter, 'last_month'); ?>>Last Month</option>
                    </select>
                    <?php submit_button('Filter', 'button', 'filter_action', false); ?>
                </div>
                <div class="alignleft actions">
                    <input type="submit" name="action" value="bulk_delete" class="button button-secondary" onclick="return confirm('Are you sure you want to delete the selected items?');" />
                    <input type="submit" name="action" value="bulk_update_alt_text" class="button button-secondary" />
                    <input type="submit" name="action" value="bulk_make_filename_alt_text" class="button button-secondary" />
                </div>
            </div>
        </form>

        <form method="post" action="">
            <?php wp_nonce_field('cmd_media_display_action', 'cmd_media_display_nonce'); ?>
            <table class="wp-list-table widefat fixed striped">
                <thead>
                    <tr>
                        <th scope="col" class="manage-column check-column">
                            <input type="checkbox" id="cb-select-all-1" />
                        </th>
                        <th scope="col" class="manage-column" style="max-width:180px;">Thumbnail</th>
                        <th scope="col" class="manage-column">Filename</th>
                        <th scope="col" class="manage-column">File Size</th>
                        <th scope="col" class="manage-column"><a href="https://www.w3schools.com/tags/att_img_alt.asp" target="_blank">Alt Text</a></th>
                        <th scope="col" class="manage-column">Related to URL</th>
                        <th scope="col" class="manage-column">Keywords (EasySEO)</th>
                        <th scope="col" class="manage-column">Date</th>
                    </tr>
                </thead>
                <tbody>
                    <?php
                    if ($query->have_posts()) {
                        foreach ($query->posts as $post) {
                            $attachment_id = $post->ID;
                            $file_name = basename(get_post_meta($attachment_id, '_wp_attached_file', true));
                            $file_type = get_post_mime_type($attachment_id);
                            $file_size = size_format(filesize(get_attached_file($attachment_id)));
                            $upload_date = get_the_date('Y-m-d H:i:s', $attachment_id);
                            $alt_text = get_post_meta($attachment_id, '_wp_attachment_image_alt', true);

                            $attached_to_id = $post->post_parent;
                            $attached_to_title = $attached_to_id ? get_the_title($attached_to_id) : '';
                            $attached_to_url = $attached_to_id ? get_permalink($attached_to_id) : '#';

                            $common_words = cmd_get_common_words($alt_text, $attached_to_title);
                            ?>
                            <tr>
                                <th scope="row" class="check-column"><input type="checkbox" name="media_ids[]" value="<?php echo esc_attr($attachment_id); ?>" /></th>
                                <?php if ($show_image) { ?>
                                    <td style="max-width:180px;"><img src="<?php echo esc_url(wp_get_attachment_url($attachment_id)); ?>" alt="" width="50" /></td>
                                <?php } else { ?>
                                    <td>&nbsp;</td>
                                <?php } ?>
                                <td>
                                    <a href="<?php echo esc_url(wp_get_attachment_url($attachment_id)); ?>" class="lightbox-link" data-lightbox="image-<?php echo esc_attr($attachment_id); ?>"><?php echo esc_html($file_name); ?></a>
                                </td>
                                
                                <td><?php echo esc_html($file_size); ?><br /><?php echo esc_html($file_type); ?></td>
                                <td>
                                    <input type="text" name="alt_text[<?php echo esc_attr($attachment_id); ?>]" value="<?php echo esc_attr($alt_text); ?>">
                                </td>
                                <td>
                                    <a href="<?php echo esc_url($attached_to_url); ?>" target="_blank"><?php echo esc_html($attached_to_title); ?></a>
                                </td>
                                <td><?php echo esc_html($common_words); ?></td>
                                <td><?php echo esc_html($upload_date); ?></td>
                            </tr>
                            <?php
                        }
                    } else {
                        ?>
                        <tr>
                            <td colspan="9">No media found.</td>
                        </tr>
                        <?php
                    }
                    ?>
                </tbody>
                <tfoot>
                    <tr>
                        <th scope="col" class="manage-column check-column">
                            <input type="checkbox" id="cb-select-all-2">
                        </th>
                        <th scope="col" class="manage-column" style="max-width:180px;">Thumbnail</th>
                        <th scope="col" class="manage-column">Filename</th>
                        <th scope="col" class="manage-column">File Size</th>
                        <th scope="col" class="manage-column">Alt Text</th>
                        <th scope="col" class="manage-column">Related to URL</th>
                        <th scope="col" class="manage-column">Keywords (EasySEO)</th>
                        <th scope="col" class="manage-column">Date</th>
                    </tr>
                </tfoot>
            </table>
            <input type="submit" class="button action" value="Delete Selected">
            <input type="submit" class="button action" name="action" value="bulk_update_alt_text">
            <input type="submit" class="button action" name="action" value="bulk_make_filename_alt_text">
        </form>

        <div class="tablenav bottom">
            <div class="alignleft actions">
                <?php
                $total_pages = ceil($total_images / $rows_per_page);
                echo paginate_links(array(
                    'total'   => $total_pages,
                    'current' => $paged,
                ));
                ?>
            </div>
        </div>
    </div>

    <!-- Lightbox HTML -->
    <div id="lightbox" style="display:none;">
        <div id="lightbox-content">
            <span id="lightbox-close">&times;</span>
            <img id="lightbox-image" src="" alt="" />
        </div>
    </div>
    <style>
        #lightbox {
            position: fixed;
            top: 0;
            left: 0;
            width: 100%;
            height: 100%;
            background: rgba(0, 0, 0, 0.8);
            display: flex;
            justify-content: center;
            align-items: center;
            z-index: 1000;
        }
        #lightbox-content {
            position: relative;
        }
        #lightbox-close {
            position: absolute;
            top: 10px;
            right: 10px;
            font-size: 2em;
            color: #fff;
            cursor: pointer;
        }
        #lightbox-image {
            max-width: 90%;
            max-height: 90%;
        }
    </style>
    <script>
        document.addEventListener('DOMContentLoaded', function() {
            var lightbox = document.getElementById('lightbox');
            var lightboxImage = document.getElementById('lightbox-image');
            var lightboxClose = document.getElementById('lightbox-close');

            document.querySelectorAll('.lightbox-link').forEach(function(link) {
                link.addEventListener('click', function(e) {
                    e.preventDefault();
                    var imageUrl = this.href;
                    lightboxImage.src = imageUrl;
                    lightbox.style.display = 'flex';
                });
            });

            lightboxClose.addEventListener('click', function() {
                lightbox.style.display = 'none';
            });

            lightbox.addEventListener('click', function(e) {
                if (e.target === lightbox) {
                    lightbox.style.display = 'none';
                }
            });
        });
    </script>
    <?php
}
?>
